
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include <Moof/Engine.hh>
#include <Moof/Exception.hh>
#include <Moof/Log.hh>
#include <Moof/Math.hh>
#include <Moof/OpenGL.hh>
#include <Moof/Settings.hh>
#include <Moof/Video.hh>

#include "GameLayer.hh"
#include "Hud.hh"

#if HAVE_CONFIG_H
#include "config.h"
#endif


void GameLayer::loadSceneLoader()
{
	mState.script.importStandardLibraries();
	importLogFunctions(mState.script);

	std::string loaderPath = Scene::getPath("loader");
	if (loaderPath == "")
	{
		throw Mf::Exception(Mf::ErrorCode::RESOURCE_NOT_FOUND, "loader");
	}

	Mf::Script::Result status = mState.script.doFile(loaderPath);
	if (status != Mf::Script::SUCCESS)
	{
		std::string str;
		mState.script[-1].get(str);

		throw Mf::Exception(Mf::ErrorCode::SCRIPT_ERROR, str);
	}

	mState.script.getGlobalTable().pushField("scenes");
	mState.script.getTop().get(mState.sceneList);
	if (mState.sceneList.size() == 0)
	{
		throw Mf::Exception(Mf::ErrorCode::SCRIPT_ERROR,
				"no variable `scenes' within loader");
	}
}

void GameLayer::advanceScene()
{
	if (mState.sceneList.size() != 0)
	{
		mState.scene = Scene::alloc(mState.sceneList[0]);
		mState.sceneList.erase(mState.sceneList.begin());

		Mf::Script::Result status = mState.scene->load(mState.script);
		if (status != Mf::Script::SUCCESS)
		{
			std::string str;
			mState.script[-1].get(str);

			throw Mf::Exception(Mf::ErrorCode::SCRIPT_ERROR, str);
		}

		mState.script.getGlobalTable().pushField("Event");
		if (mState.script[-1].isTable())
		{
			mState.script[-1].pushField("Think");
			mState.script.set("Think", Mf::Script::REGISTRY);
			mState.script.pop(2);
		}
		else
		{
			mState.script.pop();
		}
	}
}


GameLayer::GameLayer() :
	mMusic("NightFusionIntro"),
	mPunchSound("Thump")
{
	mMusic.setLooping(true);
	mMusic.enqueue("NightFusionLoop");

	bool isMute = false;
	Mf::Settings::getInstance().get("nomusic", isMute);
	if (!isMute) mMusic.play();

	//mMusic.setPosition(Mf::Vector3(10.0, 5.0, 0.0));

	loadSceneLoader();
	advanceScene();				// load the first scene

	mThinkTimer.init(boost::bind(&GameLayer::thinkTimer, this),
			0.1, Mf::Timer::REPEAT);

	mState.heroine = Heroine::alloc();
	mState.heroine->animation.startSequence("GreenDiamond");

	Mf::Scalar a[6] = {0.0, 1.5, -0.5, 3.0, -2.0, 1.0};
	mState.interp.init(a, 5.0, Mf::Interpolator::OSCILLATE);

	setProjection();
}


void GameLayer::pushed(Mf::Engine& engine)
{
	engine.push(Hud::alloc(mState));

	mRay.direction.set(1.0, 0.0);

	mLine.a.set(20, 10);
	mLine.b.set(19, 14);

	mSphere.point.set(22, 5);
	mSphere.radius = 2;

	mRayTimer.init(boost::bind(&GameLayer::rayTimer, this), 1.0, Mf::Timer::REPEAT);
}


void GameLayer::update(Mf::Engine& engine, Mf::Scalar t, Mf::Scalar dt)
{
	mState.camera.update(t, dt);
	mState.heroine->update(t, dt);

	mState.scene->checkForCollision(*mState.heroine);

	mState.camera.setPosition(Mf::Vector3(-mState.heroine->getState().position[0],
				-mState.heroine->getState().position[1], -6));
	//mState.camera.lookAt(Mf::promote(mState.heroine->getState().position));

	mRay.point = mState.heroine->getState().position;
}

void GameLayer::thinkTimer()
{
	mState.script.getRegistryTable().pushField("Think");
	if (mState.script[-1].isFunction()) mState.script.call();
	else                                mState.script.pop();
}


void GameLayer::rayTimer()
{
	Mf::Ray<2>::Intersection meh;
	std::list<Mf::Ray<2>::Intersection> hits;
	Mf::Vector2 point;

	bool bam = mLine.intersectRay(mRay, meh);
	if (bam)
	{
		meh.normal.normalize();
		hits.push_back(meh);
	}

	bam = mSphere.intersectRay(mRay, meh);
	if (bam)
	{
		meh.normal.normalize();
		hits.push_back(meh);
	}

	if (mState.scene->castRay(mRay, hits))
	{
		hits.front().normal.normalize();
		mRay.solve(point, hits.front().distance);
		Mf::logInfo << "scene: d = " << hits.front().distance << std::endl;
		Mf::logInfo << "       P = " << point << std::endl;
		Mf::logInfo << "       n = " << hits.front().normal << std::endl;
	}
}


void GameLayer::draw(Mf::Engine& engine, Mf::Scalar alpha) const
{
	mState.camera.uploadToGL(alpha);

	// DRAW THE SCENE
	Mf::Texture::resetBind();

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	mState.scene->drawIfVisible(alpha, mState.camera.getFrustum());

	mState.heroine->draw(alpha);

	mRay.draw();
	mLine.draw();
	mSphere.draw();
}

bool GameLayer::handleEvent(Mf::Engine& engine, const Mf::Event& event)
{
	switch (event.type)
	{
		case SDL_KEYDOWN:
			if (event.key.keysym.sym == SDLK_SPACE)
			{
				mState.heroine->animation.startSequence("Flattened");
				Mf::logInfo("thump!");
				mPunchSound.play();
				return true;
			}
			else if (event.key.keysym.sym == SDLK_m)
			{
				mMusic.toggle();
				return true;
			}
			else if (event.key.keysym.sym == SDLK_PAGEUP)
			{
				mRay.direction = cml::rotate_vector_2D(mRay.direction,
						cml::rad(10.0));
				return true;
			}
			else if (event.key.keysym.sym == SDLK_PAGEDOWN)
			{
				mRay.direction = cml::rotate_vector_2D(mRay.direction,
						cml::rad(-10.0));
				return true;
			}
			else if (event.key.keysym.sym == SDLK_r)
			{
				loadSceneLoader();
				advanceScene();
				return true;
			}
			return mState.heroine->handleEvent(event);

		case SDL_KEYUP:
			if (event.key.keysym.sym == SDLK_ESCAPE)
			{
				engine.pop(this);
				return true;
			}
			else if (event.key.keysym.sym == SDLK_h)
			{
				engine.push(Hud::alloc(mState));
				return true;
			}
			return mState.heroine->handleEvent(event);

		case SDL_MOUSEMOTION:
		case SDL_MOUSEBUTTONDOWN:
			mState.camera.handleEvent(event);
			return true;

		case SDL_VIDEORESIZE:
			setProjection(event.resize.w, event.resize.h);
			break;
	}

	return false;
}


void GameLayer::setProjection()
{
	Mf::VideoP video = Mf::Engine::getInstance().getVideo();
	setProjection(video->getWidth(), video->getHeight());
}

void GameLayer::setProjection(Mf::Scalar width, Mf::Scalar height)
{
	mState.camera.setProjection(cml::rad(60.0), width / height, 1.0, 200.0);
}


/** vim: set ts=4 sw=4 tw=80: *************************************************/

