
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_SPHERE_HH_
#define _MOOF_SPHERE_HH_

#include <Moof/Cullable.hh>
#include <Moof/Drawable.hh>
#include <Moof/Frustum.hh>
#include <Moof/Math.hh>
#include <Moof/OpenGL.hh>
#include <Moof/Shape.hh>


namespace Mf {


/**
 * A round object.
 */

template <int D = 3>
struct Sphere : public Cullable, public Drawable, public Shape<D>
{
	typedef cml::vector< Scalar, cml::fixed<D> >	Vector;

	// (solution - point)^2 - radius^2 = 0
	Vector	point;
	Scalar	radius;

	Sphere() {}

	Sphere(const Vector& p, Scalar r) :
		point(p),
		radius(r) {}

	//Sphere(Scalar x, Scalar y, Scalar z, Scalar r) :
		//point(x, y, z),
		//radius(r) {}

	void init(const Vector& p, Scalar r)
	{
		point = p;
		radius = r;
	}

	void init(const Vector& p, const Vector& o)
	{
		point = p;
		radius = (o - p).length();
	}


	// a ray inside the sphere will not intersect on its way out
	bool intersectRay(const Ray<D>& ray, typename Ray<D>::Intersection& hit)
	{
		Vector b = point - ray.point;
		Scalar z = cml::dot(b, ray.direction);

		// check if the ball is behind the ray
		if (z < SCALAR(0.0)) return false;

		Scalar d2 = cml::dot(b, b) - z*z;
		Scalar r2 = radius * radius;

		// check for an intersection
		if (d2 > r2) return false;

		hit.distance = z - std::sqrt(r2 - d2);
		if (hit.distance < SCALAR(0.0)) return false;

		Vector surfacePoint;
		ray.solve(surfacePoint, hit.distance);
		hit.normal = surfacePoint - point;
		return true;
	}


	//void encloseVertices(const Vector vertices[], unsigned count);

	//void draw(Scalar alpha = 0.0) const;
	//bool isVisible(const Frustum& frustum) const;

void encloseVertices(const Vector vertices[], unsigned count)
{
	// TODO
}

void draw(Scalar alpha = 0.0) const;
//{
	//GLUquadricObj* sphereObj = gluNewQuadric();
	//gluQuadricDrawStyle(sphereObj, GLU_LINE);

	//glPushMatrix();

	//glTranslate(point);
	//gluSphere(sphereObj, GLdouble(radius), 16, 16);

	//glPopMatrix();

	//gluDeleteQuadric(sphereObj);
//}

bool isVisible(const Frustum& frustum) const
{
	return true;
}
};


template <>
inline bool Sphere<3>::isVisible(const Frustum& frustum) const
{
	return frustum.contains(*this);
}

template <>
inline void Sphere<2>::draw(Scalar alpha) const
{
	GLUquadricObj* sphereObj = gluNewQuadric();
	gluQuadricDrawStyle(sphereObj, GLU_LINE);

	glPushMatrix();

	glTranslate(promote(point));
	gluSphere(sphereObj, GLdouble(radius), 16, 16);

	glPopMatrix();

	gluDeleteQuadric(sphereObj);
}

template <>
inline void Sphere<3>::draw(Scalar alpha) const
{
	GLUquadricObj* sphereObj = gluNewQuadric();
	gluQuadricDrawStyle(sphereObj, GLU_LINE);

	glPushMatrix();

	glTranslate(point);
	gluSphere(sphereObj, GLdouble(radius), 16, 16);

	glPopMatrix();

	gluDeleteQuadric(sphereObj);
}

template <int D>
inline bool checkCollision(const Sphere<D>& a, const Sphere<D>& b)
{
	Scalar d = (a.point - b.point).length();
	return d < (a.radius + b.radius);
}


} // namespace Mf

#endif // _MOOF_SPHERE_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

